/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import QtQuick.XmlListModel 2.0
import "../common/ClockUtils.js" as Utils
import "../common"

Page {
    id: clockPage;

    // Property to hold the formatted time string to show on the screen
    property string currentTimeFormatted
    property real currentTimeStamp
    property real diff: new Date().getTimezoneOffset()

    Component.onCompleted: {
        Utils.log("ClockPage loaded");
        diff = worldModel.timeDiff;
        currentLocationCity.text = worldModel.city;
        easterEggCircle.longitude = worldModel.longitude;
        easterEggCircle.latitude = worldModel.latitude;
        Utils.log("Saved Current Location: " + worldModel.city);
        Utils.log("Saved Current Location Coordinates: " + easterEggCircle.longitude + "," + easterEggCircle.latitude)
        currentTimeFormatted = Qt.formatTime(new Date(currentTimeStamp + (diff * 60000)), "hh:mm")
        updateTime();
    }

    actions: [
        Action {
            id: addCityAction
            text: i18n.tr("Add City")
            keywords: i18n.tr("Add;Timezone;Timezones;World;City;Cities;Town;Towns;Place;Places;Location;Locations;Time;Locale;Local;Current")
            description: "Add a world city"
            iconSource: Qt.resolvedUrl("../images/add_icon.png")
            onTriggered: pagestack.push(Qt.resolvedUrl("WorldPage.qml"))
        }
    ]

    function onTimerUpdate(now) {
        currentTimeFormatted = Qt.formatTime(new Date(currentTimeStamp + (diff * 60000)), "hh:mm")
        now.setMinutes(now.getMinutes() + diff)
        clockFace.timeChanged(now)
        if (now.getUTCSeconds() == 0) {
            updateTime();
        }
    }

    function updateTime() {
        var now = new Date();
        currentTimeStamp = now.getTime();
    }    

    Flickable {
        id: clockAnimationContainer

        clip: true;
        anchors.fill: parent
        contentWidth: parent.width
        // The units.gu(3) is to give some gap at the bottom between the listview and the phone bottom
        contentHeight: clockFace.height + clockFace.anchors.topMargin + savedWorldClock.height + savedWorldClock.anchors.topMargin + units.gu(3)

        // Label to show the current time
        Rectangle {
            id: labelContainer;

            z: clockFace.z + 1;
            width: clockFace.innerCircle.width; height: width;
            anchors.centerIn: clockFace;
            color: "transparent"

            Label {
                id: currentTimeLabel
                objectName: "currentTimeLabel"

                anchors.centerIn: parent
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                color: Theme.palette.normal.baseText
                font.pixelSize: units.dp(41)
                text: currentTimeFormatted
            }
        }

        // Qml Element to draw the analogue clock face along with its hour, minute and second hands.
        AnalogClockFace {
            id: clockFace

            anchors { top: parent.top; topMargin: units.gu(10); horizontalCenter: parent.horizontalCenter }

            onClicked: {
                if (easterEggCircle.isReady == XmlListModel.Ready) {
                    clockFace.state == "" ? clockFace.state = "easteregg" : clockFace.state = "";
                } else {
                    Utils.log("Sunrise/Sunset times not yet loaded.")
                }
            }

            EasterEgg {
                id: easterEggCircle
                anchors.centerIn: parent
                width: clockFace.innerCircle.width; height: width;
                radius: width / 2;
            }

            states: [
                State {
                    name: "easteregg"
                    PropertyChanges { target: easterEggCircle; opacity: 1; }
                    PropertyChanges { target: currentTimeLabel; opacity: 0; }
                    // TODO: Do some caching of data here. Store data locally to reduce API calls.
                    PropertyChanges { target: easterEggCircle; sunriseLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunriseTime); }
                    PropertyChanges { target: easterEggCircle; sunsetLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunsetTime); }
                },
                State {
                    name: ""
                    PropertyChanges { target: currentTimeLabel; opacity: 1; }
                    PropertyChanges { target: easterEggCircle; opacity: 0; }
                }
            ]
        }

        Column {
            id: savedWorldClock

            property int dynamicTopSpacing: clockPage.height - clockPage.header.height - clockFace.height - locationHeader.height - clockFace.anchors.topMargin - units.gu(1);

            height: childrenRect.height;
            anchors { left:parent.left; right:parent.right; top: clockFace.bottom; topMargin: savedWorldClock.dynamicTopSpacing }

            ListItem.ThinDivider {}

            ListItem.Header {
                id: locationHeader
                Label {
                    text: i18n.tr("Current Location")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListItem.Standard {
                id: currentLocation
                Label {
                    id: currentLocationCity
                    text: i18n.tr("UTC")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                    color: Theme.palette.normal.baseText
                    fontSize: "large"
                }
                Label {
                    text: Qt.formatTime(new Date(currentTimeStamp + (diff * 60000)), "hh:mm A")
                    anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "large"
                }
            }

            ListItem.Header {
                Label {
                    text: i18n.tr("World")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListView {
                id: listWorldClocks

                clip: true
                anchors { left: parent.left; right: parent.right }                
                height: 3 * units.gu(6) + units.gu(1)  // height is defined to show 3 items by default with a small margin
                model: worldModel
                currentIndex: -1

                delegate: ListItem.Standard {
                    Label {
                        text: cityName
                        anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }

                    Label {
                        text: Qt.formatTime(new Date(currentTimeStamp + (rawOffSet * 60000)), "hh:mm A")
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }

                    selected: listWorldClocks.currentIndex == index
                    removable: true

                    backgroundIndicator: SwipeDelete {
                        id: swipeDelete
                        state: swipingState
                    }

                    onItemRemoved: {
                        worldModel.removePreset(index);
                    }

                    onClicked: {
                        currentLocationCity.text = cityName;
                        diff = rawOffSet;
                        easterEggCircle.longitude = longitude;
                        easterEggCircle.latitude = latitude;
                        worldModel.appendCurrentLocation(cityName, diff, longitude, latitude);
                    }
                }
            }
        }
    }

    tools: ToolbarItems {
        id: toolbarClock
        ToolbarButton {
            id: addCity
            action: addCityAction
        }
    }
}
