/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0
import QtQuick.LocalStorage 2.0

import "../common/ClockUtils.js" as Utils

// Qml Element to draw the easter egg (sunrise and sunset) on the analogue clock face.
Rectangle {
    id: easterEggCircle;

    property var db: null

    // Opens the local db if not already open. On the first call creates the table
    function openDB () {
        if(db !== null) return;

        db = LocalStorage.openDatabaseSync("ubuntu-clock-app", "0.1", "Sunrise and Sunset Database", 1000);

        try {
            db.transaction(function(tx){
                tx.executeSql('CREATE TABLE IF NOT EXISTS Easteregg(key INTEGER PRIMARY KEY, date TEXT, riseTime TEXT, setTime TEXT)');
                var table  = tx.executeSql("SELECT * FROM Easteregg");
                if (table.rows.length == 0) {
                    tx.executeSql('INSERT INTO Easteregg VALUES(?, ?, ?, ?)', [1, '', '', '']);
                };
            });
        } catch (err) {
            Utils.error("Error creating table in Sunrise/Sunset database: " + err);
        };
    }

    //Read date last checked from DB
    function getSavedData () {

        openDB();

        var result = new Array();
        try {
            db.readTransaction(
                function(tx){
                    var res = tx.executeSql('SELECT * FROM Easteregg WHERE key = 1');
                    if (res.rows.length > 0) {
                        result[0] = new Array();
                        result[0][0] = res.rows.item(0).key;
                        result[0][1] = res.rows.item(0).date;
                        result[0][2] = res.rows.item(0).riseTime;
                        result[0][3] = res.rows.item(0).setTime;
                    };
                });
        } catch (err) {
            Utils.error("Error getting sunrise/sunset last checked date: " + err);
        }
        return result;
    }

    function saveSunTimes (date, sunRise, sunSet) {
        openDB();
        try {
            db.transaction(function(tx){
                var res = tx.executeSql('UPDATE Easteregg SET date = ?, riseTime = ?, setTime = ? WHERE key = 1', [date, sunRise, sunSet]);
                Utils.log("Saving Sunrise/Sunset times to disk");
            });
        } catch (err) {
            Utils.error("Error saving sunrise/sunset times: " + err);
        };
    }

    // FIXME: Replace these constant values with user's location coordinates when automatic location detection is implemented.
    property real latitude: 52.01;
    property real longitude: 4.35;

    // properties to set/retrieve the sunrise and sunset times
    property alias sunriseLabel: sunriseTimeLabel.text;
    property alias sunsetLabel: sunsetTimeLabel.text;

    // properties to get the sunRiseModel XML model.
    property alias isReady: sunRiseModel.status;
    property alias model: sunRiseModel;

    // Online API properties
    readonly property string base_url: "http://api.geonames.org/timezone?";
    readonly property string username: "krnekhelesh";

    opacity: 0
    color: "transparent"
    z: parent.z + 100
    visible: opacity == 0 ? false : true

    function reloadSunRiseModel () {
        sunRiseModel.reload()
    }

    // Function to form the url string to make the API call
    function getUrlString (latitude, longitude) {
        return (base_url + "lat=" + latitude + "&lng=" + longitude + "&username=" + username);
    }

    // Function to convert online data format "2013-04-20 19:40" to just "19:40"
    function getSunTime (data) {
        return data.split(' ')[1];
    }

    // TODO 1: Get the latitude and longitude using the platform API automatically instead of using static values.    
    XmlListModel {
        id: sunRiseModel

        source: getUrlString(latitude, longitude)
        query: "/geonames/timezone"

        XmlRole { name: "sunriseTime"; query: "sunrise/string()"; isKey: true }
        XmlRole { name: "sunsetTime"; query: "sunset/string()"; isKey: true  }

        onStatusChanged: {
            var savedData = getSavedData();
            var checkedDate = savedData[0][1].split('T')[0]
            var date = new Date().toISOString();
            if (status == XmlListModel.Ready && checkedDate != date.split('T')[0]){
                sunriseTimeLabel.text = getSunTime(sunRiseModel.get(0).sunriseTime);
                sunsetTimeLabel.text = getSunTime(sunRiseModel.get(0).sunsetTime);
                saveSunTimes(date, sunriseTimeLabel.text, sunsetTimeLabel.text);
            }
            else if (status == XmlListModel.Ready) {
                Utils.log("Retrieving Sunrise/Sunset times from disk");
                sunriseTimeLabel.text = savedData[0][2];
                sunsetTimeLabel.text = savedData[0][3];
            }
        }
    }
    
    Column {        
        anchors.centerIn: parent
        width: parent.width; height: childrenRect.height
        
        Label {
            id: sunriseLabel
            anchors.horizontalCenter: parent.horizontalCenter
            color: Theme.palette.normal.baseText
            fontSize: "medium"
            text: i18n.tr("Sunrise")
        }
        
        Label {
            id: sunriseTimeLabel
            anchors.horizontalCenter: parent.horizontalCenter
            color: Theme.palette.normal.baseText
            fontSize: "x-large"
            text: i18n.tr("Loading")
        }
        
        Rectangle {
            id: sunsetsunriseDivider
            width: parent.width - units.gu(8)
            height: units.gu(0.1)
            anchors.horizontalCenter: parent.horizontalCenter            
            color: Theme.palette.normal.baseText
        }
        
        Label {
            id: sunsetTimeLabel
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            color: Theme.palette.normal.baseText
            fontSize: "x-large"
            text: i18n.tr("Loading")
        }
        
        Label {
            id: sunsetLabel
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            color: Theme.palette.normal.baseText
            fontSize: "medium"
            text: i18n.tr("Sunset")
        }
        
    }
}
