import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import QtQuick.XmlListModel 2.0

Column {
    id: worldClocks

    // Properties to store the user searched city details
    property string city: "null";
    property real lng: 0;
    property real lat: 0;
    property real rawOffSet: 0;

    // Property to store the city searched by the user
    property string search_string: "null";

    // Properties to store the information related to api.geonames.org
    readonly property string username: "krnekhelesh";
    readonly property string search_base_url: "http://api.geonames.org/search?q=";
    readonly property string timezone_base_url: "http://api.geonames.org/timezone?lat=";
    readonly property string local_base_url: "common-cities.xml";
    readonly property int no_of_results: 5;

    property alias searchModel: searchCityModel;
    property alias index: worldList.currentIndex;

    signal clicked()

    function searchCityUrl (city) {
        return (search_base_url + search_string + "&maxRows=" + no_of_results + "&username=" + username + "&style=full");
    }

    function localCityUrl () {
        return Qt.resolvedUrl(local_base_url);
    }

    function getCityTimezoneUrl (lat, lng) {
        return (timezone_base_url + lat + "&lng=" + lng + "&username=" + username);
    }

    function clearUserSearch () {
        searchCityModel.source = localCityUrl();
        searchLabel.text = "";
        worldList.currentIndex = -1;
    }

    height: childrenRect.height;
    spacing: units.gu(2);

    // Xml model to search city online and retrieve latitude and longitude of searched city
    XmlListModel {
        id: searchCityModel;

        source: localCityUrl()
        query: "/geonames/geoname"

        XmlRole { name: "city"; query: "toponymName/string()"; isKey: true }
        XmlRole { name: "country"; query: "countryName/string()"; isKey: true }
        XmlRole { name: "lat"; query: "lat/string()"; isKey: true }
        XmlRole { name: "lng"; query: "lng/string()"; isKey: true }
        XmlRole { name: "adminName"; query: "adminName1/string()"; isKey: true}

        onSourceChanged: reload();
    }

    Item {
        id: searchBox;

        anchors { left: parent.left; right: parent.right }
        height: units.gu(4)

        TextField {
            id: searchLabel

            anchors { left: parent.left; leftMargin: units.gu(2); top: parent.top; bottom: parent.bottom; right: searchButton.left; rightMargin: -units.gu(1) }
            width: worldClocks.width/1.5
            hasClearButton: true
            placeholderText: i18n.tr("Search")
            primaryItem: Image {
                height: parent.height/2
                width: parent.height/2
                source: Qt.resolvedUrl("../images/search_icon.svg")
            }

            // Provide a small pause before going online to search
            Timer {
                id: search_timer

                interval: 1100
                repeat: false
                onTriggered: {
                    if (searchLabel.text != "") {
                        search_string = searchLabel.text;
                        searchCityModel.source = searchCityUrl(search_string);
                    } else {
                        searchCityModel.source = localCityUrl()
                    }
                }
            }

            onTextChanged: search_timer.restart()

            InverseMouseArea {
                visible: searchLabel.activeFocus
                anchors.fill: parent
                onClicked: {
                    worldClocks.forceActiveFocus()
                    mouse.accepted = false
                }
            }
        }

        Item {
            id: searchButton;

            anchors { right: parent.right; rightMargin: units.gu(6.5); top: parent.top; bottom: parent.bottom }
            height: searchLabel.height
            visible: !searchActivity.running
            Image {
                id: name
                source: Qt.resolvedUrl("../images/search_item.svg")
            }

            // Make searchButton clickable
            MouseArea {
                id: searchArea;
                enabled: true;
                anchors.fill: searchButton
                onClicked: {
                    if (searchLabel.text != "") {
                        search_string = searchLabel.text;
                        searchCityModel.source = searchCityUrl(search_string);
                    }
                }
            }
        }


        ActivityIndicator {
            id: searchActivity
            anchors { verticalCenter: searchButton.verticalCenter; left: searchButton.left; leftMargin: units.gu(1.5) }
            running: searchCityModel.status === XmlListModel.Loading

        }
    }

    ListView {
        id: worldList

        anchors { left: parent.left; right: parent.right }
        height: units.gu(50)
        model: searchCityModel
        currentIndex: -1
        clip: true;

        Component {
            id: sectionHeading
            ListItem.Header {
                Label {
                    text: section
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }
        }

        Label {
            visible:  worldList.count == 0
            anchors.horizontalCenter: parent.horizontalCenter
            text: i18n.tr("No results found")
            fontSize: "large"
        }

        section.property: "city"
        section.criteria: ViewSection.FirstCharacter
        section.delegate: sectionHeading;
        section.labelPositioning: ViewSection.InlineLabels

        delegate: ListItem.Standard {
            Label {
                id: cityDelegate
                text: city
                anchors { top: parent.top; topMargin: units.gu(0.5); left: parent.left; leftMargin: units.gu(3) }
                color: Theme.palette.normal.baseText
                fontSize: "large"
            }
            Label {
                text: adminName ? adminName + ", " + country : country
                anchors { top: cityDelegate.bottom; bottom: parent.bottom; left: parent.left; leftMargin: units.gu(3) }
                color: Theme.palette.normal.baseText
                fontSize: "small"
            }

            selected: worldList.currentIndex == index;

            onClicked: {
                worldList.currentIndex = index;
                worldClocks.clicked()
            }
        }
    }
}
