/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Themes.Ambiance 0.1
import "../common"
import "../common/ClockUtils.js" as Utils

Page {
    id: timerPage

    property alias minutes: analogTimer.minutes;
    property alias seconds: analogTimer.seconds;
    property alias totalTime: analogTimer.totalTime;
    property alias timerOn: analogTimer.timerOn;
    property string tempNamePreset
    property real tempPosition

    actions:
        Action {
        id: addTimerAction
        iconSource: Qt.resolvedUrl("../images/add_icon.png")
        text: i18n.tr("Add Timer")
        description: i18n.tr("Add a new timer")
        keywords: i18n.tr("Add;Timer;Eggtimer;Preset;Ticker")
        onTriggered: {
            reset()
            state = "addPreset"
        }
    }

    states: [
        State {
            name: "selectPreset"
            when: listTimerPreset.currentIndex != -1
            PropertyChanges { target: namePreset; visible: true }
            PropertyChanges { target: editPresetToolbarButton; visible: true }
            PropertyChanges { target: addPresetToolbarButton; visible: false }
        },
        State {
            name: "addPreset"
            PropertyChanges { target: timerMouseArea; enabled: false }
            PropertyChanges { target: listPreset; visible: false }
            PropertyChanges { target: namePreset; visible: true }
            PropertyChanges { target: addPresetToolbarButton; visible: false }
            PropertyChanges { target: editPresetToolbarButton; visible: true }
            PropertyChanges { target: cancelPresetToolbarButton; visible: true }
            PropertyChanges { target: toolbarTimer; locked: true }
            PropertyChanges { target: toolbarTimer; opened: true }
        }
    ]

    function onTimerUpdate() {
        analogTimer.onTimerUpdate();
    }

    // Function to print timer label (format mm:ss and hh:mm:ss when hh > 0) at top
    function intervalToString(minutes, seconds)
    {
        return Utils.zeroleft(minutes, 2) + ":" + Utils.zeroleft(seconds, 2);
    }

    function reset() {
        analogTimer.reset();
        listTimerPreset.currentIndex = -1;
        namePreset.text = "";
        tempNamePreset = "";
    }

    // Function to calculate and print the minutes and seconds from the total time as a string
    function getstringTimer(time) {
        var minutes, seconds;

        seconds = time % 60
        time = Math.floor(time / 60)
        minutes = time % 60

        return Utils.zeroleft(minutes, 2) + ":" + Utils.zeroleft(seconds, 2);
    }

    Component.onCompleted: {
        Utils.log("TimerPage loaded");
        tempPosition = timerAnimationContainer.contentY
    }

    Flickable {
        id: timerAnimationContainer

        clip: true;
        anchors.fill: parent
        interactive: presetModel.count != 0 ? true : false;
        contentWidth: parent.width
        // The units.gu(3) is to give some gap at the bottom between the listview and the phone bottom
        contentHeight: analogTimer.height + analogTimer.anchors.topMargin + listPreset.height + listPreset.anchors.topMargin + units.gu(2)

        Behavior on contentY {
            UbuntuNumberAnimation { duration: UbuntuAnimation.SlowDuration }
        }

        // Label to show the current time
        Item {
            id: labelContainer;

            z: analogTimer.z + 1;
            width: analogTimer.innerCircle.width; height: width;
            anchors.centerIn: analogTimer;

            Label {
                id: labelTimer
                objectName: "labelTimer"

                anchors.centerIn: parent
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                color: Theme.palette.normal.baseText

                fontSize: "x-large"
                text: labelTimer.text = intervalToString(minutes, seconds);
            }
        }

        // Main element which draws the timer face and manages time.
        AnalogTimer {
            id: analogTimer

            Timer {
                id: seconds_timer

                interval: 500
                repeat: true
                onTriggered: labelTimer.text = intervalToString(minutes, seconds);
            }

            AnalogGlowEffect {
                id: glowEffect

                z: parent.z - 1
                visible: false;
                fillRectangle: parent
            }

            anchors { top: parent.top; topMargin: units.gu(10); horizontalCenter: parent.horizontalCenter }
            inProgressFlag: (timerOn == true || analogTimer.state == "DONE") ? true : false;
            onSecondsChanged: if (timerOn == true) seconds_timer.restart()
                              else labelTimer.text = intervalToString(minutes, seconds);

            onMinutesChanged: labelTimer.text = intervalToString(minutes, seconds);

            states:[
                State { name: "STOP" },
                State { name: "PAUSE"},
                State {
                    name: "DONE"
                    when: (totalTime == -1 && timerOn == false)
                    PropertyChanges { target: glowEffect; visible: true }
                    PropertyChanges { target: glowEffect; color: "#03CB4A" }
                }
            ]

            MouseArea {
                id: timerMouseArea
                anchors.fill: analogTimer.innerCircle
                enabled: true
                onClicked: {
                    switch (analogTimer.state) {

                    case "":
                        if ((seconds != 0 || minutes != 0) || timerOn) {
                            totalTime = (minutes * 60) + seconds;
                            analogTimer.startTime = new Date();
                            timerOn = true;
                            analogTimer.state = "STOP";
                        }
                        break;

                    case "STOP":
                        timerOn = false;
                        analogTimer.pauseTime = analogTimer.totalTime - analogTimer.remTime + 1; // +1 is necessary to start immediately the timer when finish a pause
                        analogTimer.state = "PAUSE";
                        break;

                    case "DONE":
                        reset();
                        analogTimer.state = "";
                        break;

                    case "PAUSE":
                        analogTimer.startTime = new Date();
                        timerOn = true;
                        analogTimer.state = "STOP";
                        break;
                    }
                }
            }
        }

        // Element to set the name of a saved preset.
        TextField {
            id: namePreset
            objectName: "namePreset"

            text: "";
            anchors { horizontalCenter: analogTimer.horizontalCenter; top: analogTimer.bottom; topMargin: units.gu(2) }
            width: analogTimer.width/2;
            style: TextFieldStyle {
                background: Item {}
            }
            // #FIXME: The anchoring needs to be done properly
            primaryItem: Image {
                fillMode: Image.PreserveAspectFit
                source: Qt.resolvedUrl("../images/edit_icon.png")
            }
            color: Theme.palette.normal.baseText
            visible: false
            hasClearButton: false
            horizontalAlignment: TextInput.AlignHCenter
            placeholderText: i18n.tr("Preset Name")
            font.pixelSize: FontUtils.sizeToPixels("large")

            InverseMouseArea {
                visible: namePreset.activeFocus
                anchors.fill: parent
                onClicked: {
                    analogTimer.forceActiveFocus()
                    mouse.accepted = false
                }
            }
        }

        Image {
            id: inputDotImage
            visible:namePreset.focus
            source: Qt.resolvedUrl("../images/dot.svg")
            fillMode: Image.Tile
            anchors { top: namePreset.bottom; left: namePreset.left; right: namePreset.right }
        }

        ImageButton {
            id: resetButtonTimer
            objectName: "resetButtonTimer"

            anchors { top: analogTimer.bottom; topMargin: units.gu(7); horizontalCenter: analogTimer.horizontalCenter; leftMargin: units.gu(4) }
            buttonImage: Qt.resolvedUrl("../images/reset_icon.svg");
            buttonLabel: i18n.tr("Reset");
            visible: timerOn || (minutes != 0 || seconds != 0)? true : false;
            enabled: true

            onClicked: {
                reset();
                if (timerPage.state != "addPreset")
                    timerAnimationContainer.contentY = tempPosition

            }
        }

        // Column element to hold the saved presets
        Column {
            id: listPreset

            property int dynamicTopSpacing: timerPage.height - timerPage.header.height - analogTimer.height - locationHeader.height - analogTimer.anchors.topMargin - units.gu(1);

            height: childrenRect.height
            visible: presetModel.count != 0 ? true : false
            anchors { left: parent.left; right: parent.right; top: analogTimer.bottom; topMargin: listPreset.dynamicTopSpacing }

            ListItem.ThinDivider { }

            ListItem.Header {
                id: locationHeader
                Label {
                    text: i18n.tr("Presets")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListView {
                id: listTimerPreset

                clip: true
                anchors { left: parent.left; right: parent.right }
                height: 5 * units.gu(6) + units.gu(1)
                model: presetModel
                currentIndex: -1

                delegate: ListItem.Standard {
                    objectName: "preset" + index
                    Label {
                        id: presetTextLabel // Needs by autopilot
                        objectName: "presetTextLabel" + index
                        fontSize: "large";
                        text: label
                        anchors { verticalCenter: parent.verticalCenter; left: presetTimeLabel.right; leftMargin: units.gu(5) }
                        color: Theme.palette.normal.baseText
                    }

                    Label {
                        id: presetTimeLabel
                        objectName: "presetTimerLabel"
                        fontSize: "large"
                        text: getstringTimer(seconds);
                        anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                        color: Theme.palette.normal.baseText
                    }

                    selected: listTimerPreset.currentIndex == index
                    removable: true

                    backgroundIndicator: SwipeDelete {
                        id: swipeDelete
                        state: swipingState
                    }

                    // TODO: Implement different based actions based on the swipe direction.
                    onItemRemoved: {
                        presetModel.removePreset(index);
                        if (presetModel.count == 0)
                            timerAnimationContainer.contentY = tempPosition
                    }

                    onClicked: {
                        if (!timerOn) {
                            analogTimer.ssToTime(seconds);
                            listTimerPreset.currentIndex = index
                            timerAnimationContainer.contentY = tempPosition
                            namePreset.text = label;
                            tempNamePreset = label;
                        }
                    }
                }
            }
        }
    }

    // Various toolbar actions such as add preset, edit and back.
    tools: ToolbarItems {
        id: toolbarTimer

        opened: false
        locked: false

        ToolbarButton {
            id: editPresetToolbarButton
            objectName: "savePresetButton"
            visible: false
            iconSource: Qt.resolvedUrl("../images/select_icon.png")
            text: i18n.tr("Save")
            onTriggered: {
                presetModel.appendPreset(tempNamePreset, namePreset.text, ((minutes * 60) + seconds), listTimerPreset.currentIndex)
                namePreset.focus = false
                if (listTimerPreset.currentIndex == -1) {
                    reset()
                    timerPage.state = "";
                    timerAnimationContainer.contentY = tempPosition
                }
                toolbarTimer.opened = false;
            }
        }

        ToolbarButton {
            id: addPresetToolbarButton
            objectName: "addPresetButton"
            action: addTimerAction
        }

        ToolbarButton {
            id: cancelPresetToolbarButton
            objectName: "cancelPresetButton"
            visible: false;
            iconSource: Qt.resolvedUrl("../images/cancel_icon.svg")
            text: i18n.tr("Cancel")
            onTriggered: {
                reset()
                timerPage.state = "";
                toolbarTimer.opened = false;
                timerAnimationContainer.contentY = tempPosition
            }
        }
    }
}
